@@ -262,4 +262,18 @@
 # define CLUTTER_AVAILABLE_IN_1_12
 #endif
 
+#if CLUTTER_VERSION_MIN_REQUIRED >= CLUTTER_VERSION_1_14
+# define CLUTTER_DEPRECATED_IN_1_14             CLUTTER_DEPRECATED
+# define CLUTTER_DEPRECATED_IN_1_14_FOR(f)      CLUTTER_DEPRECATED_FOR(f)
+#else
+# define CLUTTER_DEPRECATED_IN_1_14
+# define CLUTTER_DEPRECATED_IN_1_14_FOR(f)
+#endif
+
+#if CLUTTER_VERSION_MAX_ALLOWED < CLUTTER_VERSION_1_14
+# define CLUTTER_AVAILABLE_IN_1_14              CLUTTER_UNAVAILABLE(1, 14)
+#else
+# define CLUTTER_AVAILABLE_IN_1_14
+#endif
+
 #endif /* __CLUTTER_MACROS_H__ */
