@@ -46,14 +46,14 @@
  *   of overriding the #ClutterEffect virtual functions, to chain up to the
  *   #ClutterOffscreenEffect's implementation.</para>
  *   <para>On top of the #ClutterEffect's virtual functions,
- *   #ClutterOffscreenEffect also provides a <function>paint_target()</function>
+ *   #ClutterOffscreenEffect also provides a #ClutterOffscreenEffectClass.paint_target()
  *   function, which encapsulates the effective painting of the texture that
  *   contains the result of the offscreen redirection.</para>
  *   <para>The size of the target material is defined to be as big as the
  *   transformed size of the #ClutterActor using the offscreen effect.
  *   Sub-classes of #ClutterOffscreenEffect can change the texture creation
  *   code to provide bigger textures by overriding the
- *   <function>create_texture()</function> virtual function; no chain up
+ *   #ClutterOffscreenEffectClass.create_texture() virtual function; no chain up
  *   to the #ClutterOffscreenEffect implementation is required in this
  *   case.</para>
  * </refsect2>
@@ -65,6 +65,8 @@
 #include "config.h"
 #endif
 
+#define CLUTTER_ENABLE_EXPERIMENTAL_API
+
 #include "clutter-offscreen-effect.h"
 
 #include "cogl/cogl.h"
@@ -77,7 +79,8 @@
 struct _ClutterOffscreenEffectPrivate
 {
   CoglHandle offscreen;
-  CoglMaterial *target;
+  CoglPipeline *target;
+  CoglHandle texture;
 
   ClutterActor *actor;
   ClutterActor *stage;
@@ -85,10 +88,6 @@ struct _ClutterOffscreenEffectPrivate
   gfloat x_offset;
   gfloat y_offset;
 
-  /* The size of the texture */
-  gfloat target_width;
-  gfloat target_height;
-
   /* This is the calculated size of the fbo before being passed
      through create_texture(). This needs to be tracked separately so
      that we can detect when a different size is calculated and
@@ -125,10 +124,10 @@ clutter_offscreen_effect_set_actor (ClutterActorMeta *meta,
   meta_class->set_actor (meta, actor);
 
   /* clear out the previous state */
-  if (priv->offscreen != COGL_INVALID_HANDLE)
+  if (priv->offscreen != NULL)
     {
       cogl_handle_unref (priv->offscreen);
-      priv->offscreen = COGL_INVALID_HANDLE;
+      priv->offscreen = NULL;
     }
 
   /* we keep a back pointer here, to avoid going through the ActorMeta */
@@ -150,7 +149,6 @@ update_fbo (ClutterEffect *effect, int fbo_width, int fbo_height)
 {
   ClutterOffscreenEffect *self = CLUTTER_OFFSCREEN_EFFECT (effect);
   ClutterOffscreenEffectPrivate *priv = self->priv;
-  CoglHandle texture;
 
   priv->stage = clutter_actor_get_stage (priv->actor);
   if (priv->stage == NULL)
@@ -164,53 +162,52 @@ update_fbo (ClutterEffect *effect, int fbo_width, int fbo_height)
 
   if (priv->fbo_width == fbo_width &&
       priv->fbo_height == fbo_height &&
-      priv->offscreen != COGL_INVALID_HANDLE)
+      priv->offscreen != NULL)
     return TRUE;
 
-  if (priv->target == COGL_INVALID_HANDLE)
+  if (priv->target == NULL)
     {
-      priv->target = cogl_material_new ();
+      CoglContext *ctx =
+        clutter_backend_get_cogl_context (clutter_get_default_backend ());
+
+      priv->target = cogl_pipeline_new (ctx);
 
       /* We're always going to render the texture at a 1:1 texel:pixel
          ratio so we can use 'nearest' filtering to decrease the
          effects of rounding errors in the geometry calculation */
-      cogl_material_set_layer_filters (priv->target,
+      cogl_pipeline_set_layer_filters (priv->target,
                                        0, /* layer_index */
-                                       COGL_MATERIAL_FILTER_NEAREST,
-                                       COGL_MATERIAL_FILTER_NEAREST);
+                                       COGL_PIPELINE_FILTER_NEAREST,
+                                       COGL_PIPELINE_FILTER_NEAREST);
     }
 
-  texture =
+  if (priv->texture != NULL)
+    {
+      cogl_handle_unref (priv->texture);
+      priv->texture = NULL;
+    }
+
+  priv->texture =
     clutter_offscreen_effect_create_texture (self, fbo_width, fbo_height);
-  if (texture == COGL_INVALID_HANDLE)
-     return FALSE;
+  if (priv->texture == NULL)
+    return FALSE;
 
-  cogl_material_set_layer (priv->target, 0, texture);
-  cogl_handle_unref (texture);
-
-  /* we need to use the size of the texture target and not the minimum
-   * size we passed to the create_texture() vfunc, as any sub-class might
-   * give use a bigger texture
-   */
-  priv->target_width = cogl_texture_get_width (texture);
-  priv->target_height = cogl_texture_get_height (texture);
+  cogl_pipeline_set_layer_texture (priv->target, 0, priv->texture);
 
   priv->fbo_width = fbo_width;
   priv->fbo_height = fbo_height;
 
-  if (priv->offscreen != COGL_INVALID_HANDLE)
+  if (priv->offscreen != NULL)
     cogl_handle_unref (priv->offscreen);
 
-  priv->offscreen = cogl_offscreen_new_to_texture (texture);
-  if (priv->offscreen == COGL_INVALID_HANDLE)
+  priv->offscreen = cogl_offscreen_new_to_texture (priv->texture);
+  if (priv->offscreen == NULL)
     {
       g_warning ("%s: Unable to create an Offscreen buffer", G_STRLOC);
 
       cogl_handle_unref (priv->target);
-      priv->target = COGL_INVALID_HANDLE;
+      priv->target = NULL;
 
-      priv->target_width = 0;
-      priv->target_height = 0;
       priv->fbo_width = 0;
       priv->fbo_height = 0;
 
@@ -231,6 +228,7 @@ clutter_offscreen_effect_pre_paint (ClutterEffect *effect)
   gfloat fbo_width, fbo_height;
   gfloat width, height;
   gfloat xexpand, yexpand;
+  int texture_width, texture_height;
 
   if (!clutter_actor_meta_get_enabled (CLUTTER_ACTOR_META (effect)))
     return FALSE;
@@ -261,6 +259,9 @@ clutter_offscreen_effect_pre_paint (ClutterEffect *effect)
   if (!update_fbo (effect, fbo_width, fbo_height))
     return FALSE;
 
+  texture_width = cogl_texture_get_width (priv->texture);
+  texture_height = cogl_texture_get_height (priv->texture);
+
   /* get the current modelview matrix so that we can copy it to the
    * framebuffer. We also store the matrix that was last used when we
    * updated the FBO so that we can detect when we don't need to
@@ -284,14 +285,14 @@ clutter_offscreen_effect_pre_paint (ClutterEffect *effect)
   xexpand = 0.f;
   if (priv->x_offset < 0.f)
     xexpand = -priv->x_offset;
-  if (priv->x_offset + priv->target_width > width)
-    xexpand = MAX (xexpand, (priv->x_offset + priv->target_width) - width);
+  if (priv->x_offset + texture_width > width)
+    xexpand = MAX (xexpand, (priv->x_offset + texture_width) - width);
 
   yexpand = 0.f;
   if (priv->y_offset < 0.f)
     yexpand = -priv->y_offset;
-  if (priv->y_offset + priv->target_height > height)
-    yexpand = MAX (yexpand, (priv->y_offset + priv->target_height) - height);
+  if (priv->y_offset + texture_height > height)
+    yexpand = MAX (yexpand, (priv->y_offset + texture_height) - height);
 
   /* Set the viewport */
   cogl_set_viewport (-(priv->x_offset + xexpand), -(priv->y_offset + yexpand),
@@ -346,7 +347,7 @@ clutter_offscreen_effect_real_paint_target (ClutterOffscreenEffect *effect)
 
   paint_opacity = clutter_actor_get_paint_opacity (priv->actor);
 
-  cogl_material_set_color4ub (priv->target,
+  cogl_pipeline_set_color4ub (priv->target,
                               paint_opacity,
                               paint_opacity,
                               paint_opacity,
@@ -359,8 +360,8 @@ clutter_offscreen_effect_real_paint_target (ClutterOffscreenEffect *effect)
    * hadn't been redirected offscreen.
    */
   cogl_rectangle_with_texture_coords (0, 0,
-                                      priv->target_width,
-                                      priv->target_height,
+                                      cogl_texture_get_width (priv->texture),
+                                      cogl_texture_get_height (priv->texture),
                                       0.0, 0.0,
                                       1.0, 1.0);
 }
@@ -396,8 +397,8 @@ clutter_offscreen_effect_post_paint (ClutterEffect *effect)
   ClutterOffscreenEffect *self = CLUTTER_OFFSCREEN_EFFECT (effect);
   ClutterOffscreenEffectPrivate *priv = self->priv;
 
-  if (priv->offscreen == COGL_INVALID_HANDLE ||
-      priv->target == COGL_INVALID_HANDLE ||
+  if (priv->offscreen == NULL ||
+      priv->target == NULL ||
       priv->actor == NULL)
     return;
 
@@ -448,6 +449,9 @@ clutter_offscreen_effect_finalize (GObject *gobject)
   if (priv->target)
     cogl_handle_unref (priv->target);
 
+  if (priv->texture)
+    cogl_handle_unref (priv->texture);
+
   G_OBJECT_CLASS (clutter_offscreen_effect_parent_class)->finalize (gobject);
 }
 
@@ -480,6 +484,35 @@ clutter_offscreen_effect_init (ClutterOffscreenEffect *self)
                                             ClutterOffscreenEffectPrivate);
 }
 
+/**
+ * clutter_offscreen_effect_get_texture:
+ * @effect: a #ClutterOffscreenEffect
+ *
+ * Retrieves the texture used as a render target for the offscreen
+ * buffer created by @effect
+ *
+ * You should only use the returned texture when painting. The texture
+ * may change after ClutterEffect::pre_paint is called so the effect
+ * implementation should update any references to the texture after
+ * chaining-up to the parent's pre_paint implementation. This can be
+ * used instead of clutter_offscreen_effect_get_target() when the
+ * effect subclass wants to paint using its own material.
+ *
+ * Return value: (transfer none): a #CoglHandle or %COGL_INVALID_HANDLE. The
+ *   returned texture is owned by Clutter and it should not be
+ *   modified or freed
+ *
+ * Since: 1.10
+ */
+CoglHandle
+clutter_offscreen_effect_get_texture (ClutterOffscreenEffect *effect)
+{
+  g_return_val_if_fail (CLUTTER_IS_OFFSCREEN_EFFECT (effect),
+                        NULL);
+
+  return effect->priv->texture;
+}
+
 /**
  * clutter_offscreen_effect_get_target:
  * @effect: a #ClutterOffscreenEffect
@@ -500,9 +533,9 @@ CoglMaterial *
 clutter_offscreen_effect_get_target (ClutterOffscreenEffect *effect)
 {
   g_return_val_if_fail (CLUTTER_IS_OFFSCREEN_EFFECT (effect),
-                        COGL_INVALID_HANDLE);
+                        NULL);
 
-  return effect->priv->target;
+  return (CoglMaterial *)effect->priv->target;
 }
 
 /**
@@ -541,7 +574,7 @@ clutter_offscreen_effect_create_texture (ClutterOffscreenEffect *effect,
                                          gfloat                  height)
 {
   g_return_val_if_fail (CLUTTER_IS_OFFSCREEN_EFFECT (effect),
-                        COGL_INVALID_HANDLE);
+                        NULL);
 
   return CLUTTER_OFFSCREEN_EFFECT_GET_CLASS (effect)->create_texture (effect,
                                                                       width,
@@ -574,17 +607,17 @@ clutter_offscreen_effect_get_target_size (ClutterOffscreenEffect *effect,
   ClutterOffscreenEffectPrivate *priv;
 
   g_return_val_if_fail (CLUTTER_IS_OFFSCREEN_EFFECT (effect), FALSE);
-  
+
   priv = effect->priv;
 
-  if (priv->target == NULL)
+  if (priv->texture == NULL)
     return FALSE;
 
   if (width)
-    *width = priv->target_width;
+    *width = cogl_texture_get_width (priv->texture);
 
   if (height)
-    *height = priv->target_height;
+    *height = cogl_texture_get_height (priv->texture);
 
   return TRUE;
 }
